package org.signwriting;

import com.dtai.gwt.GadgetGraphics;
import java.awt.Dimension;
import java.awt.Point;
import java.util.Hashtable;

public class SignSymbol extends SignComponent {
    
    static final long serialVersionUID = 4927791797228843442L;
    
    public static Hashtable hash1;
    public static Hashtable hash2;
    
    private static final int[] start = { 97, 103, 112, 120, 127, 134, 142, 146, 151, 155};
    private static final int[] end   = { 97, 108, 114, 123, 129, 134, 144, 148, 153, 157};
    private static final int[] more  = {100, 107, 108, 121, 122, 123};
    
    static {
        hash1 = new Hashtable();
        int i;
        for (int j = 0; j < start.length; j++) {
            for (i = start[j]; i <= end[j]; i++) {
                hash1.put(new Integer(i), new Integer(i));
            }
        }
        hash2 = new Hashtable();
        for (int k = 0; k < more.length; k++) {
            hash2.put(new Integer(more[k]), new Integer(more[k]));
        }
    }

    private int code;
    private boolean flop;
    private byte rot;
    private byte ahead;
    private byte fill;
    private int centerx = 0;
    private int centery = 0;

    public SignSymbol(int code, int opt, byte y, byte x) {
        this(code, opt, y, x, false);
    }
    
    public SignSymbol(int code, int opt, byte y, byte x, boolean version4_0) {
        int _rot      = (opt & 0x7);
        int _ahead    = Math.max(0,((opt >> 3) & 0x3) - 1);
        int _fill     = ((opt >> 5) & 0x3) + ((((opt >> 3) & 0x3) == 0)? 4 : 0);
        boolean _flop = ((opt>>7) == 1);  
           
        if (version4_0) {
            if (hash1.contains(new Integer(code))) {
                int temp = _fill;
                _fill = _ahead;
                _ahead = temp;
                boolean changedArrow = false;
                if (_ahead == 1) {
                    _ahead = 0;
                    changedArrow = true;
                }
                if (!changedArrow && _ahead == 0) {
                    _ahead = 1;
                }
            }
            else if (code == 183 || code == 184) {
                _fill -= 1;
                if (_fill < 0) _fill += 4;
            }
            
            if (_flop && _rot > 0 && code != 132) {
                _rot = Symbol.getSymbol(code).getNumRotations(_ahead, _fill) - _rot;
            }
            
        }
        init(code, _rot, _ahead, _fill, _flop, (x*2), (y*2));
    }

    public SignSymbol(int code, int rot, int ahead, int fill, boolean flop) {
        this(code, rot, ahead, fill, flop, 0, 0);
    }

    public SignSymbol(int code, int rot, int ahead, int fill, boolean flop,
                      int x, int y) {
        init(code, rot, ahead, fill, flop, x, y);
    }
    
    private void init(int code, int rot, int ahead, int fill, boolean flop,
                      int x, int y) {
        this.code = code;
        this.rot = (byte)rot;
        this.flop = flop;
        this.ahead = (byte)ahead;
        this.fill = (byte)fill;
        this.x = x;
        this.y = y;
        ImageCoords coords = Symbol.getSymbol(code).getCoords(ahead,
                                              this.fill, this.rot,
                                              this.flop, false);
        if (coords != null) {
            centerx = coords.centerPtH;
            centery = coords.centerPtV;
        }

        resetSize();
    }

    public SignSymbol(SignSymbol rhs) {
        super(rhs);
        code = rhs.code;
        flop = rhs.flop;
        rot = rhs.rot;
        ahead = rhs.ahead;
        fill = rhs.fill;
        centerx = rhs.centerx;
        centery = rhs.centery;
    }

    public Object clone() {
        SignSymbol newSignSymbol = new SignSymbol(this);
        return newSignSymbol;
    }

    public int hashCode() {
        return code + (rot << 8) + (ahead << 11) + (fill << 13) + ((flop?1:0) << 15);
    }

    public boolean equals(Object obj) {
        if (!super.equals(obj)) {
            return false;
        }
        SignSymbol rhs = (SignSymbol)obj;
        if (rhs.code != code ||
            rhs.flop != flop ||
            rhs.rot != rot ||
            rhs.ahead != ahead ||
            rhs.fill != fill) {
            return false;
        }
        return true;
    }

    public int getNumVariations() {
        return Symbol.getSymbol(code).getNumVariations();
    }

    public int getNumFills() {
        return Symbol.getSymbol(code).getNumFills(ahead);
    }

    public int getNumRotations() {
        return Symbol.getSymbol(code).getNumRotations(ahead, fill);
    }

    public int getNumFlops() {
        return Symbol.getSymbol(code).getNumFlops(ahead, fill);
    }

    public int getCode() {
        return code;
    } 
    
    public int getRot() {
        return rot;
    } 
    
    public void setRot(int i) {
        rot = (byte)i;
    }
    
    public void setRotation(int i) {
        rot = (byte)i;
        //WHY?
        resetSize();
    }
    
    public void setFill(int i) {
        fill = (byte)i;
        //resetSize();
    }

    public void resetSize() {
        if (ahead >= getNumVariations()) {
            ahead = 0;
        }
        if (fill >= getNumFills()) {
            fill = 0;
        }
        if (rot >= getNumRotations()) {
            rot = 0;
        } else if (rot < 0) {
            rot = (byte)(getNumRotations()-1);
        }
        if (getNumFlops() == 0) {
            //flop = false; //NO!NO!NO!
        }
        ImageCoords coords = Symbol.getSymbol(code).getCoords(
                                              ahead, this.fill, this.rot,
                                              this.flop, false);    

        x += centerx - coords.centerPtH;
        y += centery - coords.centerPtV;
        
        centerx = coords.centerPtH;
        centery = coords.centerPtV;
        super.resetSize();
    }
    
    protected int calcWidth() {
        ImageCoords coords = Symbol.getSymbol(code).getCoords(
                                              ahead, fill, rot,
                                              flop, false);
        return coords.framePtH;
    }
    
    protected int calcHeight() {
        ImageCoords coords = Symbol.getSymbol(code).getCoords(
                                              ahead, fill, rot,
                                              flop, false);
        return coords.framePtV;
    }

    public void incrementRotation(boolean clockwise) {
        if ( (clockwise && flop) || !(clockwise || flop) ) {
            rot++;
        } else {
            rot--;
            if (rot < 0) {
                rot += getNumRotations();
            }
        }
        resetSize();
    }

    public void incrementFill(boolean clockwise) {
        if (clockwise) {
            fill--;
            if (fill < 0) {
                fill += getNumFills();
            }
        }
        else {
            fill++;
        }
        resetSize();
    }

    public void changeFlop() {
        flop = !flop;
        resetSize();
    }

    public void incrementVariation(boolean clockwise) {
        if (clockwise) {
            ahead--;
            if (ahead < 0) {
                ahead += getNumVariations();
            }
        }
        else {
            ahead++;
        }
        resetSize();
    }

    public void paint(GadgetGraphics g, int x, int y, int offsetx, int offsety,
                      Bitmap bitmap, int[] pixels, PaintInfo paintInfo) {
        int xoffset = this.x;
        int yoffset = this.y;
        boolean thumbNail = false;
        if (g.getFont() == SignFont.small) {
            thumbNail = true;
            xoffset = ((xoffset) / 2);
            yoffset = ((yoffset) / 2);
        }
        Symbol.getSymbol(code).paint(g,
                     x+xoffset,y+yoffset,ahead,fill,rot,flop,thumbNail,
                     bitmap,xoffset+offsetx,yoffset+offsety, pixels, paintInfo );
    }

    public String toString() {
        return "code="+code+" y="+y+" x="+x+
               " flop="+flop+" rot="+rot+" arrowhead="+ahead+" fill="+fill;
    }


}
