package org.signwriting;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import com.dtai.gwt.LabelGadget;

/**
 * reads the symbol file
 */
public class SymbolFile {

    String filename;

    /**
     * constructor, takes a symbol file name, opens it and reads it
     */
    public SymbolFile() {
        filename = Resources.getResources().getString("SymbolFile.file");
    }

    public void readSymbols(LabelGadget label) throws IOException {

        DataInputStream in = new DataInputStream(
                             new BufferedInputStream(
                             new FileInputStream(filename)));


        in.skip(2); // skip size of short -- number of defined symbols
        
        String prefix = label.getLabel();
        int percent = 0;

    	for (int symCode = 0; symCode < Symbol.SGRID_SYMS-1; symCode++) {
    	    
    	    int newPercent = (int)(((double)symCode /
    	                            Symbol.SGRID_SYMS) * 100.0);
            if ((newPercent - percent) > 9 && ((newPercent % 10) == 0)) {
                percent = newPercent;
                label.setLabel(prefix+" ("+percent+"%)");
                label.getShell().flushGraphics();
            }
            
    		/**
    		 **	Read symbol length to determine how many bytes
    		 **	to read.  Allocate corresponding buffer and read
    		 ** symbol bytes into it.  Assign buffer to symbol
    		 ** data structure.
    		 **
    		 **/

    		int symBytes = in.readUnsignedShort();
    		if (symBytes != 0) {
    			int oldavail = in.available();
    			Symbol symbol = new Symbol();
    			symbol.readSymbolData(symCode,in);
if (symCode >= SignWriter.testStart && symCode <= SignWriter.testEnd &&
    SignWriter.allSymbolsText != null &&
    (Symbol.getSymbol(symCode) != null)) {
        
    SignWriter.allSymbolsText.append("\n");
    SignWriter.allSymbolsText.append(Integer.toString(symCode));
    SignWriter.allSymbolsText.append(": ");

boolean first = true;

int vars = 1;
vars = Symbol.getSymbol(symCode).getNumVariations();
    for (int i = 0; i < vars; i++) {
        if (i > 0) {
            SignWriter.allSymbolsText.append("\n");
        }
int fills = 1;
fills = Symbol.getSymbol(symCode).getNumFills(i);
        for (int j = 0; j < fills; j++) {
            if (j > 0) {
                SignWriter.allSymbolsText.append("\n");
            }
int rots = 1;
rots = Symbol.getSymbol(symCode).getNumRotations(i, j);
            for (int k = 0; k < rots; k++) {
                for (int l = 0; l < Symbol.getSymbol(symCode).getNumFlops(i, j)+1; l++) {
                    try {
                        SignSymbol signSym = new SignSymbol(symCode, k, i, j, l==1);
                        Sign sign = new Sign();
                        sign.add(signSym);
                        SignFont.add(sign);
                        SignWriter.allSymbolsText.append(sign.getFontIndex());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }
}

    			if ((oldavail-in.available()) != symBytes) {
    			    throw new IOException("Error reading symbol.  Expected to "+
    			        "read "+symBytes+" bytes, but read "+(oldavail-in.available())+
    			        " bytes");
    			}
    		}

    	}

        in.close();
    }
}

